"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
function* searchLocations(baseDir, resource) {
    let next = baseDir;
    let prev = '';
    while (prev !== next && path.basename(next) !== 'node_modules') {
        yield path.join(next, resource);
        prev = next;
        next = path.join(next, '../');
    }
}
function resolveResource(baseDir, fileName) {
    for (const location of searchLocations(baseDir, fileName)) {
        if (fs.existsSync(location)) {
            return location;
        }
    }
    return undefined;
}
function removeComments(source) {
    return source.replace(/\/\/.*|\/\*[^]*?\*\//g, ' ');
}
function readJSON(location) {
    let file = location;
    if (!fs.existsSync(file) && !file.endsWith('.json'))
        file += '.json';
    const content = fs.readFileSync(file, 'utf8');
    const json = removeComments(content);
    return JSON.parse(json);
}
function loadConfig(location, files = new Set) {
    let config;
    try {
        config = readJSON(location);
    }
    catch (err) {
        console.error(`Failed to parse tsconfig located at ${location}.\n
Is your configuration file properly formatted JSON?. Error message was:
\n${err.message}`);
    }
    if (config == null || typeof config !== 'object')
        return {};
    if (config.extends == null)
        return config;
    const parent = path.resolve(path.dirname(location), config.extends);
    config.extends = null;
    if (files.has(parent)) {
        console.error(`Cyclic references detected at ${location}.`);
        return config;
    }
    files.add(parent);
    const parentConfig = loadConfig(parent, files);
    return Object.assign(parentConfig, config, {
        compilerOptions: Object.assign(parentConfig.compilerOptions, config.compilerOptions),
    });
}
function resolveConfig(baseDir) {
    const location = resolveResource(baseDir, 'tsconfig.json');
    if (location === undefined)
        return {};
    const { compilerOptions } = loadConfig(location);
    return compilerOptions === undefined ? {} : compilerOptions;
}
exports.resolveConfig = resolveConfig;
function resolveTranspiler(baseDir) {
    const location = resolveResource(baseDir, 'node_modules');
    try {
        const transpiler = require(`${location}/typescript`);
        return transpiler;
    }
    catch (err) {
        console.error(`Failed to load transpiler for directory ${baseDir}.\n
Do you have TypeScript installed as a peerDependency? Error message was:
\n${err.message}`);
        return undefined;
    }
}
exports.resolveTranspiler = resolveTranspiler;
